var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __markAsModule = (target) => __defProp(target, "__esModule", { value: true });
var __export = (target, all) => {
  __markAsModule(target);
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __reExport = (target, module2, desc) => {
  if (module2 && typeof module2 === "object" || typeof module2 === "function") {
    for (let key of __getOwnPropNames(module2))
      if (!__hasOwnProp.call(target, key) && key !== "default")
        __defProp(target, key, { get: () => module2[key], enumerable: !(desc = __getOwnPropDesc(module2, key)) || desc.enumerable });
  }
  return target;
};
var __toModule = (module2) => {
  return __reExport(__markAsModule(__defProp(module2 != null ? __create(__getProtoOf(module2)) : {}, "default", module2 && module2.__esModule && "default" in module2 ? { get: () => module2.default, enumerable: true } : { value: module2, enumerable: true })), module2);
};
__export(exports, {
  createServer: () => createServer
});
var import_body_parser = __toModule(require("body-parser"));
var import_cors = __toModule(require("cors"));
var import_express = __toModule(require("express"));
var uuid = __toModule(require("uuid"));
var import_errors = __toModule(require("../errors"));
var import_cognitoLocal_public = __toModule(require("../keys/cognitoLocal.public.json"));
var import_pino_http = __toModule(require("pino-http"));
const createServer = (router, logger, options = {}) => {
  const pino = (0, import_pino_http.default)({
    logger,
    useLevel: "debug",
    genReqId: () => uuid.v4().split("-")[0],
    quietReqLogger: true,
    autoLogging: {
      ignore: (req) => req.method === "OPTIONS"
    }
  });
  const app = (0, import_express.default)();
  app.use(pino);
  app.use((0, import_cors.default)({
    origin: "*"
  }));
  app.use(import_body_parser.default.json({
    type: "application/x-amz-json-1.1"
  }));
  app.get("/:userPoolId/.well-known/jwks.json", (req, res) => {
    res.status(200).json({
      keys: [import_cognitoLocal_public.default.jwk]
    });
  });
  app.get("/:userPoolId/.well-known/openid-configuration", (req, res) => {
    res.status(200).json({
      id_token_signing_alg_values_supported: ["RS256"],
      jwks_uri: `http://localhost:9229/${req.params.userPoolId}/.well-known/jwks.json`,
      issuer: `http://localhost:9229/${req.params.userPoolId}`
    });
  });
  app.get("/health", (req, res) => {
    res.status(200).json({ ok: true });
  });
  app.post("/", (req, res) => {
    const xAmzTarget = req.headers["x-amz-target"];
    if (!xAmzTarget) {
      res.status(400).json({ message: "Missing x-amz-target header" });
      return;
    } else if (xAmzTarget instanceof Array) {
      res.status(400).json({ message: "Too many x-amz-target headers" });
      return;
    }
    const [, target] = xAmzTarget.split(".");
    if (!target) {
      res.status(400).json({ message: "Invalid x-amz-target header" });
      return;
    }
    const route = router(target);
    const replacer = function(key, value) {
      if (this[key] instanceof Date) {
        return Math.floor(this[key].getTime() / 1e3);
      }
      return value;
    };
    route({ logger: req.log }, req.body).then((output) => res.status(200).type("json").send(JSON.stringify(output, replacer)), (ex) => {
      if (ex instanceof import_errors.UnsupportedError) {
        if (options.development) {
          req.log.info("======");
          req.log.info("");
          req.log.info("Unsupported target");
          req.log.info("");
          req.log.info(`x-amz-target: ${xAmzTarget}`);
          req.log.info("Body:");
          req.log.info(JSON.stringify(req.body, void 0, 2));
          req.log.info("");
          req.log.info("======");
        }
        req.log.error(`Cognito Local unsupported feature: ${ex.message}`);
        res.status(500).json({
          __type: "CognitoLocal#Unsupported",
          message: `Cognito Local unsupported feature: ${ex.message}`
        });
        return;
      } else if (ex instanceof import_errors.CognitoError) {
        req.log.warn(ex, `Error handling target: ${target}`);
        res.status(400).json({
          __type: ex.code,
          message: ex.message
        });
        return;
      } else {
        req.log.error(ex, `Error handling target: ${target}`);
        res.status(500).json(ex);
        return;
      }
    });
  });
  return {
    application: app,
    start(startOptions) {
      const actualOptions = __spreadValues(__spreadValues({
        port: (options == null ? void 0 : options.port) ?? 9229,
        hostname: (options == null ? void 0 : options.hostname) ?? "localhost",
        development: (options == null ? void 0 : options.development) ?? false
      }, options), startOptions);
      return new Promise((resolve, reject) => {
        const httpServer = app.listen(actualOptions.port, actualOptions.hostname, () => resolve(httpServer));
        httpServer.on("error", reject);
      });
    }
  };
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  createServer
});
