import { UserPoolDefaults } from "../server/config";
import { AppClient } from "./appClient";
import { Clock } from "./clock";
import { Context } from "./context";
import { DataStore } from "./dataStore/dataStore";
import { DataStoreFactory } from "./dataStore/factory";
import { UserPool, UserPoolService, UserPoolServiceFactory } from "./userPoolService";
export declare const USER_POOL_AWS_DEFAULTS: UserPoolDefaults;
export interface CognitoService {
    createUserPool(ctx: Context, userPool: UserPool): Promise<UserPool>;
    deleteUserPool(ctx: Context, userPool: UserPool): Promise<void>;
    getAppClient(ctx: Context, clientId: string): Promise<AppClient | null>;
    getUserPool(ctx: Context, userPoolId: string): Promise<UserPoolService>;
    getUserPoolForClientId(ctx: Context, clientId: string): Promise<UserPoolService>;
    listAppClients(ctx: Context, userPoolId: string): Promise<readonly AppClient[]>;
    listUserPools(ctx: Context): Promise<readonly UserPool[]>;
}
export interface CognitoServiceFactory {
    create(ctx: Context, userPoolDefaultConfig: UserPoolDefaults): Promise<CognitoService>;
}
export declare class CognitoServiceImpl implements CognitoService {
    private readonly clients;
    private readonly clock;
    private readonly userPoolServiceFactory;
    private readonly dataDirectory;
    private readonly userPoolDefaultConfig;
    constructor(dataDirectory: string, clients: DataStore, clock: Clock, userPoolDefaultConfig: UserPoolDefaults, userPoolServiceFactory: UserPoolServiceFactory);
    createUserPool(ctx: Context, userPool: UserPool): Promise<UserPool>;
    deleteUserPool(ctx: Context, userPool: UserPool): Promise<void>;
    getUserPool(ctx: Context, userPoolId: string): Promise<UserPoolService>;
    getUserPoolForClientId(ctx: Context, clientId: string): Promise<UserPoolService>;
    getAppClient(ctx: Context, clientId: string): Promise<AppClient | null>;
    listAppClients(ctx: Context, userPoolId: string): Promise<readonly AppClient[]>;
    listUserPools(ctx: Context): Promise<readonly UserPool[]>;
}
export declare class CognitoServiceFactoryImpl implements CognitoServiceFactory {
    private readonly dataDirectory;
    private readonly clock;
    private readonly dataStoreFactory;
    private readonly userPoolServiceFactory;
    constructor(dataDirectory: string, clock: Clock, dataStoreFactory: DataStoreFactory, userPoolServiceFactory: UserPoolServiceFactory);
    create(ctx: Context, userPoolDefaultConfig: UserPoolDefaults): Promise<CognitoService>;
}
