var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __markAsModule = (target) => __defProp(target, "__esModule", { value: true });
var __export = (target, all) => {
  __markAsModule(target);
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __reExport = (target, module2, desc) => {
  if (module2 && typeof module2 === "object" || typeof module2 === "function") {
    for (let key of __getOwnPropNames(module2))
      if (!__hasOwnProp.call(target, key) && key !== "default")
        __defProp(target, key, { get: () => module2[key], enumerable: !(desc = __getOwnPropDesc(module2, key)) || desc.enumerable });
  }
  return target;
};
var __toModule = (module2) => {
  return __reExport(__markAsModule(__defProp(module2 != null ? __create(__getProtoOf(module2)) : {}, "default", module2 && module2.__esModule && "default" in module2 ? { get: () => module2.default, enumerable: true } : { value: module2, enumerable: true })), module2);
};
__export(exports, {
  StormDBDataStore: () => StormDBDataStore,
  StormDBDataStoreFactory: () => StormDBDataStoreFactory
});
var import_fs = __toModule(require("fs"));
var import_stormdb = __toModule(require("stormdb"));
var import_util = __toModule(require("util"));
class StormDBDataStore {
  constructor(db) {
    this.db = db;
  }
  async delete(ctx, key) {
    ctx.logger.debug({ key }, "DataStore.delete");
    (key instanceof Array ? key : [key]).reduce((acc, k) => acc.get([k]), this.db).delete(false);
    ctx.logger.debug({ store: this.db.value() }, "DataStore.save");
    await this.db.save();
  }
  async getRoot(ctx) {
    ctx.logger.debug("DataStore.getRoot");
    return await this.db.value() ?? null;
  }
  async get(ctx, key, defaultValue) {
    ctx.logger.debug({ key }, "DataStore.get");
    return await (key instanceof Array ? key : [key]).reduce((acc, k) => acc.get([k]), this.db).value() ?? defaultValue ?? null;
  }
  async set(ctx, key, value) {
    ctx.logger.debug({ key, value }, "DataStore.set");
    this.db.setValue(value, key instanceof Array ? key : [key]);
    ctx.logger.debug({ store: this.db.value() }, "DataStore.save");
    await this.db.save();
  }
}
const mkdir = (0, import_util.promisify)(import_fs.default.mkdir);
const replaceDatesWithISOStrings = function(key, value) {
  if (!key.endsWith("Date")) {
    return value;
  }
  const val = this[key];
  if (!(val instanceof Date)) {
    throw new Error(`Serialize: Expected ${key} field to contain a Date, received a ${typeof this[key]}`);
  }
  return val.toISOString();
};
const reviveDates = (key, value) => {
  if (!key.endsWith("Date")) {
    return value;
  }
  if (typeof value === "number") {
    return new Date(value);
  }
  if (typeof value === "string") {
    return new Date(Date.parse(value));
  }
  throw new Error(`Deserialize: Expected ${key} to contain a String or Number, received a ${typeof value}`);
};
const createStormDBInstance = (directory, id) => {
  const engine = new import_stormdb.default.localFileEngine(`${directory}/${id}.json`, {
    async: true,
    serialize: (obj) => JSON.stringify(obj, replaceDatesWithISOStrings, 2),
    deserialize: (obj) => JSON.parse(obj, reviveDates)
  });
  return new import_stormdb.default(engine);
};
class StormDBDataStoreFactory {
  constructor(directory, dataStoreCache) {
    this.directory = directory;
    this.cache = dataStoreCache;
  }
  async create(ctx, id, defaults) {
    ctx.logger.debug({ id }, "createDataStore");
    await mkdir(this.directory, { recursive: true });
    const cachedDb = this.cache.get(id);
    if (cachedDb) {
      ctx.logger.debug({ id }, "Using cached data store");
      return cachedDb;
    }
    ctx.logger.debug({ id }, "Creating new data store");
    const db = createStormDBInstance(this.directory, id);
    db.default(defaults);
    ctx.logger.debug({ store: db.value() }, "DataStore.save");
    await db.save();
    const dataStore = new StormDBDataStore(db);
    this.cache.set(id, dataStore);
    return dataStore;
  }
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  StormDBDataStore,
  StormDBDataStoreFactory
});
