var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __markAsModule = (target) => __defProp(target, "__esModule", { value: true });
var __export = (target, all) => {
  __markAsModule(target);
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __reExport = (target, module2, desc) => {
  if (module2 && typeof module2 === "object" || typeof module2 === "function") {
    for (let key of __getOwnPropNames(module2))
      if (!__hasOwnProp.call(target, key) && key !== "default")
        __defProp(target, key, { get: () => module2[key], enumerable: !(desc = __getOwnPropDesc(module2, key)) || desc.enumerable });
  }
  return target;
};
var __toModule = (module2) => {
  return __reExport(__markAsModule(__defProp(module2 != null ? __create(__getProtoOf(module2)) : {}, "default", module2 && module2.__esModule && "default" in module2 ? { get: () => module2.default, enumerable: true } : { value: module2, enumerable: true })), module2);
};
__export(exports, {
  LambdaService: () => LambdaService
});
var import_package = __toModule(require("aws-sdk/package.json"));
var import_errors = __toModule(require("../errors"));
class LambdaService {
  constructor(config, lambdaClient) {
    this.config = config;
    this.lambdaClient = lambdaClient;
  }
  enabled(lambda) {
    return !!this.config[lambda];
  }
  async invoke(ctx, trigger, event) {
    const functionName = this.config[trigger];
    if (!functionName) {
      throw new Error(`${trigger} trigger not configured`);
    }
    const lambdaEvent = this.createLambdaEvent(event);
    ctx.logger.debug({
      functionName,
      event: JSON.stringify(lambdaEvent, void 0, 2)
    }, `Invoking "${functionName}" with event`);
    let result;
    try {
      result = await this.lambdaClient.invoke({
        FunctionName: functionName,
        InvocationType: "RequestResponse",
        Payload: JSON.stringify(lambdaEvent)
      }).promise();
    } catch (ex) {
      ctx.logger.error(ex);
      throw new import_errors.UnexpectedLambdaExceptionError();
    }
    ctx.logger.debug(`Lambda completed with StatusCode=${result.StatusCode} and FunctionError=${result.FunctionError}`);
    if (!result.FunctionError) {
      try {
        const parsedPayload = JSON.parse(result.Payload);
        return parsedPayload.response;
      } catch (err) {
        ctx.logger.error(err);
        throw new import_errors.InvalidLambdaResponseError();
      }
    } else {
      ctx.logger.error({ result }, result.FunctionError);
      if (result.FunctionError === "Unhandled" && result.Payload) {
        const parsedPayload = JSON.parse(result.Payload);
        if (parsedPayload.errorMessage) {
          throw new import_errors.UserLambdaValidationError(`${functionName} failed with error ${parsedPayload.errorMessage}.`);
        }
      }
      throw new import_errors.UserLambdaValidationError(result.FunctionError);
    }
  }
  createLambdaEvent(event) {
    const version = "0";
    const callerContext = {
      awsSdkVersion: import_package.version,
      clientId: event.clientId
    };
    const region = "local";
    switch (event.triggerSource) {
      case "PostAuthentication_Authentication": {
        return {
          version,
          callerContext,
          region,
          userPoolId: event.userPoolId,
          triggerSource: event.triggerSource,
          userName: event.username,
          request: {
            userAttributes: event.userAttributes,
            clientMetadata: event.clientMetadata,
            newDeviceUsed: false
          },
          response: {}
        };
      }
      case "PostConfirmation_ConfirmForgotPassword":
      case "PostConfirmation_ConfirmSignUp": {
        return {
          version,
          callerContext,
          region,
          userPoolId: event.userPoolId,
          triggerSource: event.triggerSource,
          userName: event.username,
          request: {
            userAttributes: event.userAttributes,
            clientMetadata: event.clientMetadata
          },
          response: {}
        };
      }
      case "PreSignUp_AdminCreateUser":
      case "PreSignUp_ExternalProvider":
      case "PreSignUp_SignUp": {
        return {
          version,
          callerContext,
          region,
          userPoolId: event.userPoolId,
          triggerSource: event.triggerSource,
          userName: event.username,
          request: {
            userAttributes: event.userAttributes,
            clientMetadata: event.clientMetadata,
            validationData: event.validationData
          },
          response: {
            autoConfirmUser: false,
            autoVerifyEmail: false,
            autoVerifyPhone: false
          }
        };
      }
      case "TokenGeneration_AuthenticateDevice":
      case "TokenGeneration_Authentication":
      case "TokenGeneration_HostedAuth":
      case "TokenGeneration_NewPasswordChallenge":
      case "TokenGeneration_RefreshTokens": {
        return {
          version,
          callerContext,
          region,
          userPoolId: event.userPoolId,
          triggerSource: event.triggerSource,
          userName: event.username,
          request: {
            userAttributes: event.userAttributes,
            groupConfiguration: {},
            clientMetadata: event.clientMetadata
          },
          response: {
            claimsOverrideDetails: {}
          }
        };
      }
      case "UserMigration_Authentication": {
        return {
          version,
          callerContext,
          region,
          userPoolId: event.userPoolId,
          triggerSource: event.triggerSource,
          userName: event.username,
          request: {
            clientMetadata: event.clientMetadata,
            password: event.password,
            validationData: event.validationData
          },
          response: {
            desiredDeliveryMediums: [],
            finalUserStatus: void 0,
            forceAliasCreation: void 0,
            messageAction: void 0,
            userAttributes: {}
          }
        };
      }
      case "CustomMessage_SignUp":
      case "CustomMessage_AdminCreateUser":
      case "CustomMessage_ResendCode":
      case "CustomMessage_ForgotPassword":
      case "CustomMessage_UpdateUserAttribute":
      case "CustomMessage_VerifyUserAttribute":
      case "CustomMessage_Authentication": {
        return {
          version,
          callerContext,
          region,
          userPoolId: event.userPoolId,
          triggerSource: event.triggerSource,
          userName: event.username,
          request: {
            clientMetadata: event.clientMetadata,
            codeParameter: event.codeParameter,
            usernameParameter: event.usernameParameter,
            userAttributes: event.userAttributes
          },
          response: {
            smsMessage: "",
            emailMessage: "",
            emailSubject: ""
          }
        };
      }
      case "CustomEmailSender_SignUp":
      case "CustomEmailSender_ResendCode":
      case "CustomEmailSender_ForgotPassword":
      case "CustomEmailSender_UpdateUserAttribute":
      case "CustomEmailSender_VerifyUserAttribute":
      case "CustomEmailSender_AdminCreateUser":
        return {
          version,
          region,
          userPoolId: event.userPoolId,
          triggerSource: event.triggerSource,
          userName: event.username,
          callerContext,
          request: {
            type: "customEmailSenderRequestV1",
            code: event.code,
            userAttributes: event.userAttributes,
            clientMetadata: event.clientMetadata
          },
          response: {}
        };
      default: {
        throw new Error("Unsupported Trigger Source");
      }
    }
  }
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  LambdaService
});
