import { AttributeListType } from "aws-sdk/clients/cognitoidentityserviceprovider";
import { CustomMessageTriggerResponse, Lambda } from "../lambda";
import { Trigger } from "./trigger";
export declare type CustomMessageTrigger = Trigger<{
    source: "CustomMessage_SignUp" | "CustomMessage_AdminCreateUser" | "CustomMessage_ResendCode" | "CustomMessage_ForgotPassword" | "CustomMessage_UpdateUserAttribute" | "CustomMessage_VerifyUserAttribute" | "CustomMessage_Authentication";
    userPoolId: string;
    clientId: string | null;
    username: string;
    code: string;
    userAttributes: AttributeListType;
    /**
     * One or more key-value pairs that you can provide as custom input to the Lambda function that you specify for the
     * custom message trigger. You can pass this data to your Lambda function by using the ClientMetadata parameter in the
     * following API actions:
     *
     * - AdminResetUserPassword
     * - AdminRespondToAuthChallenge
     * - AdminUpdateUserAttributes
     * - ForgotPassword
     * - GetUserAttributeVerificationCode
     * - ResendConfirmationCode
     * - SignUp
     * - UpdateUserAttributes
     *
     * Source: https://docs.aws.amazon.com/cognito/latest/developerguide/user-pool-lambda-custom-message.html#cognito-user-pools-lambda-trigger-syntax-custom-message
     */
    clientMetadata: Record<string, string> | undefined;
}, CustomMessageTriggerResponse | null>;
interface CustomMessageServices {
    lambda: Lambda;
}
export declare const CustomMessage: ({ lambda }: CustomMessageServices) => CustomMessageTrigger;
export {};
