var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __markAsModule = (target) => __defProp(target, "__esModule", { value: true });
var __export = (target, all) => {
  __markAsModule(target);
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __reExport = (target, module2, desc) => {
  if (module2 && typeof module2 === "object" || typeof module2 === "function") {
    for (let key of __getOwnPropNames(module2))
      if (!__hasOwnProp.call(target, key) && key !== "default")
        __defProp(target, key, { get: () => module2[key], enumerable: !(desc = __getOwnPropDesc(module2, key)) || desc.enumerable });
  }
  return target;
};
var __toModule = (module2) => {
  return __reExport(__markAsModule(__defProp(module2 != null ? __create(__getProtoOf(module2)) : {}, "default", module2 && module2.__esModule && "default" in module2 ? { get: () => module2.default, enumerable: true } : { value: module2, enumerable: true })), module2);
};
__export(exports, {
  AdminCreateUser: () => AdminCreateUser
});
var import_short_uuid = __toModule(require("short-uuid"));
var uuid = __toModule(require("uuid"));
var import_errors = __toModule(require("../errors"));
var import_userPoolService = __toModule(require("../services/userPoolService"));
var import_responses = __toModule(require("./responses"));
const generator = (0, import_short_uuid.default)("0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz!");
const selectAppropriateDeliveryMethod = (desiredDeliveryMediums, user) => {
  if (desiredDeliveryMediums.includes("SMS")) {
    const phoneNumber = (0, import_userPoolService.attributeValue)("phone_number", user.Attributes);
    if (phoneNumber) {
      return {
        AttributeName: "phone_number",
        DeliveryMedium: "SMS",
        Destination: phoneNumber
      };
    }
  }
  if (desiredDeliveryMediums.includes("EMAIL")) {
    const email = (0, import_userPoolService.attributeValue)("email", user.Attributes);
    if (email) {
      return {
        AttributeName: "email",
        DeliveryMedium: "EMAIL",
        Destination: email
      };
    }
  }
  return null;
};
const deliverWelcomeMessage = async (ctx, req, temporaryPassword, user, messages, userPool) => {
  const deliveryDetails = selectAppropriateDeliveryMethod(req.DesiredDeliveryMediums ?? ["SMS"], user);
  if (!deliveryDetails) {
    throw new import_errors.InvalidParameterError("User has no attribute matching desired delivery mediums");
  }
  await messages.deliver(ctx, "AdminCreateUser", null, userPool.options.Id, user, temporaryPassword, req.ClientMetadata, deliveryDetails);
};
const AdminCreateUser = ({
  clock,
  cognito,
  messages,
  config
}) => async (ctx, req) => {
  var _a;
  const userPool = await cognito.getUserPool(ctx, req.UserPoolId);
  const existingUser = await userPool.getUserByUsername(ctx, req.Username);
  const supressWelcomeMessage = req.MessageAction === "SUPPRESS";
  if (existingUser && req.MessageAction === "RESEND") {
    throw new import_errors.UnsupportedError("AdminCreateUser with MessageAction=RESEND");
  } else if (existingUser) {
    throw new import_errors.UsernameExistsError();
  }
  const attributes = (0, import_userPoolService.attributesInclude)("sub", req.UserAttributes) ? req.UserAttributes ?? [] : [{ Name: "sub", Value: uuid.v4() }, ...req.UserAttributes ?? []];
  const now = clock.get();
  const temporaryPassword = req.TemporaryPassword ?? process.env.CODE ?? generator.new().slice(0, 6);
  const isEmailUsername = (_a = config.UserPoolDefaults.UsernameAttributes) == null ? void 0 : _a.includes("email");
  const hasEmailAttribute = (0, import_userPoolService.attributesInclude)("email", attributes);
  if (isEmailUsername && !hasEmailAttribute) {
    attributes.push({ Name: "email", Value: req.Username });
  }
  const user = {
    Username: req.Username,
    Password: temporaryPassword,
    Attributes: attributes,
    Enabled: true,
    UserStatus: "FORCE_CHANGE_PASSWORD",
    ConfirmationCode: void 0,
    UserCreateDate: now,
    UserLastModifiedDate: now,
    RefreshTokens: []
  };
  await userPool.saveUser(ctx, user);
  if (!supressWelcomeMessage) {
    await deliverWelcomeMessage(ctx, req, temporaryPassword, user, messages, userPool);
  }
  return {
    User: (0, import_responses.userToResponseObject)(user)
  };
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  AdminCreateUser
});
