var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __markAsModule = (target) => __defProp(target, "__esModule", { value: true });
var __export = (target, all) => {
  __markAsModule(target);
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __reExport = (target, module2, desc) => {
  if (module2 && typeof module2 === "object" || typeof module2 === "function") {
    for (let key of __getOwnPropNames(module2))
      if (!__hasOwnProp.call(target, key) && key !== "default")
        __defProp(target, key, { get: () => module2[key], enumerable: !(desc = __getOwnPropDesc(module2, key)) || desc.enumerable });
  }
  return target;
};
var __toModule = (module2) => {
  return __reExport(__markAsModule(__defProp(module2 != null ? __create(__getProtoOf(module2)) : {}, "default", module2 && module2.__esModule && "default" in module2 ? { get: () => module2.default, enumerable: true } : { value: module2, enumerable: true })), module2);
};
__export(exports, {
  AdminInitiateAuth: () => AdminInitiateAuth
});
var import_errors = __toModule(require("../errors"));
const adminUserPasswordAuthFlow = async (ctx, services, req) => {
  if (!req.AuthParameters) {
    throw new import_errors.InvalidParameterError("Missing required parameter authParameters");
  }
  if (!req.AuthParameters.USERNAME || !req.AuthParameters.PASSWORD) {
    throw new import_errors.InvalidParameterError("AuthParameters USERNAME and PASSWORD are required");
  }
  const userPool = await services.cognito.getUserPoolForClientId(ctx, req.ClientId);
  const userPoolClient = await services.cognito.getAppClient(ctx, req.ClientId);
  let user = await userPool.getUserByUsername(ctx, req.AuthParameters.USERNAME);
  if (!user && services.triggers.enabled("UserMigration")) {
    user = await services.triggers.userMigration(ctx, {
      clientMetadata: {},
      validationData: {},
      userPoolId: userPool.options.Id,
      clientId: req.ClientId,
      username: req.AuthParameters.USERNAME,
      password: req.AuthParameters.PASSWORD,
      userAttributes: []
    });
  }
  if (!user || !userPoolClient) {
    throw new import_errors.NotAuthorizedError();
  }
  if (user.Password !== req.AuthParameters.PASSWORD) {
    throw new import_errors.InvalidPasswordError();
  }
  if (user.UserStatus === "UNCONFIRMED") {
    throw new import_errors.UserNotConfirmedException();
  }
  const userGroups = await userPool.listUserGroupMembership(ctx, user);
  const tokens = await services.tokenGenerator.generate(ctx, user, userGroups, userPoolClient, req.ClientMetadata, "Authentication");
  await userPool.storeRefreshToken(ctx, tokens.RefreshToken, user);
  return {
    ChallengeName: void 0,
    Session: void 0,
    ChallengeParameters: void 0,
    AuthenticationResult: {
      AccessToken: tokens.AccessToken,
      RefreshToken: tokens.RefreshToken,
      IdToken: tokens.IdToken,
      NewDeviceMetadata: void 0,
      TokenType: void 0,
      ExpiresIn: void 0
    }
  };
};
const refreshTokenAuthFlow = async (ctx, services, req) => {
  if (!req.AuthParameters) {
    throw new import_errors.InvalidParameterError("Missing required parameter authParameters");
  }
  if (!req.AuthParameters.REFRESH_TOKEN) {
    throw new import_errors.InvalidParameterError("AuthParameters REFRESH_TOKEN is required");
  }
  const userPool = await services.cognito.getUserPoolForClientId(ctx, req.ClientId);
  const userPoolClient = await services.cognito.getAppClient(ctx, req.ClientId);
  const user = await userPool.getUserByRefreshToken(ctx, req.AuthParameters.REFRESH_TOKEN);
  if (!user || !userPoolClient) {
    throw new import_errors.NotAuthorizedError();
  }
  const userGroups = await userPool.listUserGroupMembership(ctx, user);
  const tokens = await services.tokenGenerator.generate(ctx, user, userGroups, userPoolClient, req.ClientMetadata, "RefreshTokens");
  return {
    ChallengeName: void 0,
    Session: void 0,
    ChallengeParameters: void 0,
    AuthenticationResult: {
      AccessToken: tokens.AccessToken,
      RefreshToken: void 0,
      IdToken: tokens.IdToken,
      NewDeviceMetadata: void 0,
      TokenType: void 0,
      ExpiresIn: void 0
    }
  };
};
const AdminInitiateAuth = (services) => async (ctx, req) => {
  if (req.AuthFlow === "ADMIN_USER_PASSWORD_AUTH") {
    return adminUserPasswordAuthFlow(ctx, services, req);
  } else if (req.AuthFlow === "REFRESH_TOKEN_AUTH" || req.AuthFlow === "REFRESH_TOKEN") {
    return refreshTokenAuthFlow(ctx, services, req);
  } else {
    throw new import_errors.UnsupportedError(`AdminInitAuth with AuthFlow=${req.AuthFlow}`);
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  AdminInitiateAuth
});
