var __create = Object.create;
var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
var __markAsModule = (target) => __defProp(target, "__esModule", { value: true });
var __export = (target, all) => {
  __markAsModule(target);
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __reExport = (target, module2, desc) => {
  if (module2 && typeof module2 === "object" || typeof module2 === "function") {
    for (let key of __getOwnPropNames(module2))
      if (!__hasOwnProp.call(target, key) && key !== "default")
        __defProp(target, key, { get: () => module2[key], enumerable: !(desc = __getOwnPropDesc(module2, key)) || desc.enumerable });
  }
  return target;
};
var __toModule = (module2) => {
  return __reExport(__markAsModule(__defProp(module2 != null ? __create(__getProtoOf(module2)) : {}, "default", module2 && module2.__esModule && "default" in module2 ? { get: () => module2.default, enumerable: true } : { value: module2, enumerable: true })), module2);
};
__export(exports, {
  InitiateAuth: () => InitiateAuth
});
var import_uuid = __toModule(require("uuid"));
var import_errors = __toModule(require("../errors"));
var import_userPoolService = __toModule(require("../services/userPoolService"));
const verifyMfaChallenge = async (ctx, user, req, userPool, services) => {
  var _a, _b;
  if (!((_a = user.MFAOptions) == null ? void 0 : _a.length)) {
    throw new import_errors.NotAuthorizedError();
  }
  const smsMfaOption = (_b = user.MFAOptions) == null ? void 0 : _b.find((x) => x.DeliveryMedium === "SMS");
  if (!smsMfaOption) {
    throw new import_errors.UnsupportedError("MFA challenge without SMS");
  }
  const deliveryDestination = (0, import_userPoolService.attributeValue)(smsMfaOption.AttributeName, user.Attributes);
  if (!deliveryDestination) {
    throw new import_errors.UnsupportedError(`SMS_MFA without ${smsMfaOption.AttributeName}`);
  }
  const code = services.otp();
  await services.messages.deliver(ctx, "Authentication", req.ClientId, userPool.options.Id, user, code, req.ClientMetadata, {
    DeliveryMedium: smsMfaOption.DeliveryMedium,
    AttributeName: smsMfaOption.AttributeName,
    Destination: deliveryDestination
  });
  await userPool.saveUser(ctx, __spreadProps(__spreadValues({}, user), {
    MFACode: code
  }));
  return {
    ChallengeName: "SMS_MFA",
    ChallengeParameters: {
      CODE_DELIVERY_DELIVERY_MEDIUM: "SMS",
      CODE_DELIVERY_DESTINATION: deliveryDestination,
      USER_ID_FOR_SRP: user.Username
    }
  };
};
const verifyPasswordChallenge = async (ctx, user, req, userPool, userPoolClient, services) => {
  const userGroups = await userPool.listUserGroupMembership(ctx, user);
  const tokens = await services.tokenGenerator.generate(ctx, user, userGroups, userPoolClient, void 0, "Authentication");
  await userPool.storeRefreshToken(ctx, tokens.RefreshToken, user);
  return {
    ChallengeName: "PASSWORD_VERIFIER",
    ChallengeParameters: {},
    AuthenticationResult: tokens
  };
};
const newPasswordChallenge = (user) => ({
  ChallengeName: "NEW_PASSWORD_REQUIRED",
  ChallengeParameters: {
    USER_ID_FOR_SRP: user.Username,
    requiredAttributes: JSON.stringify([]),
    userAttributes: JSON.stringify((0, import_userPoolService.attributesToRecord)(user.Attributes))
  },
  Session: (0, import_uuid.v4)()
});
const userPasswordAuthFlow = async (ctx, req, userPool, userPoolClient, services) => {
  if (!req.AuthParameters) {
    throw new import_errors.InvalidParameterError("Missing required parameter authParameters");
  }
  let user = await userPool.getUserByUsername(ctx, req.AuthParameters.USERNAME);
  if (!user && services.triggers.enabled("UserMigration")) {
    user = await services.triggers.userMigration(ctx, {
      clientId: req.ClientId,
      password: req.AuthParameters.PASSWORD,
      userAttributes: [],
      username: req.AuthParameters.USERNAME,
      userPoolId: userPool.options.Id,
      clientMetadata: void 0,
      validationData: req.ClientMetadata
    });
  }
  if (!user) {
    throw new import_errors.NotAuthorizedError();
  }
  if (user.UserStatus === "RESET_REQUIRED") {
    throw new import_errors.PasswordResetRequiredError();
  }
  if (user.UserStatus === "FORCE_CHANGE_PASSWORD") {
    return newPasswordChallenge(user);
  }
  if (user.Password !== req.AuthParameters.PASSWORD) {
    throw new import_errors.InvalidPasswordError();
  }
  if (user.UserStatus === "UNCONFIRMED") {
    throw new import_errors.UserNotConfirmedException();
  }
  if (userPool.options.MfaConfiguration === "OPTIONAL" && (user.MFAOptions ?? []).length > 0 || userPool.options.MfaConfiguration === "ON") {
    return verifyMfaChallenge(ctx, user, req, userPool, services);
  }
  if (services.triggers.enabled("PostAuthentication")) {
    await services.triggers.postAuthentication(ctx, {
      clientId: req.ClientId,
      clientMetadata: void 0,
      source: "PostAuthentication_Authentication",
      userAttributes: user.Attributes,
      username: user.Username,
      userPoolId: userPool.options.Id
    });
  }
  return verifyPasswordChallenge(ctx, user, req, userPool, userPoolClient, services);
};
const refreshTokenAuthFlow = async (ctx, req, userPool, userPoolClient, services) => {
  if (!req.AuthParameters) {
    throw new import_errors.InvalidParameterError("Missing required parameter authParameters");
  }
  if (!req.AuthParameters.REFRESH_TOKEN) {
    throw new import_errors.InvalidParameterError("AuthParameters REFRESH_TOKEN is required");
  }
  const user = await userPool.getUserByRefreshToken(ctx, req.AuthParameters.REFRESH_TOKEN);
  if (!user) {
    throw new import_errors.NotAuthorizedError();
  }
  const userGroups = await userPool.listUserGroupMembership(ctx, user);
  const tokens = await services.tokenGenerator.generate(ctx, user, userGroups, userPoolClient, void 0, "RefreshTokens");
  return {
    ChallengeName: void 0,
    Session: void 0,
    ChallengeParameters: void 0,
    AuthenticationResult: {
      AccessToken: tokens.AccessToken,
      RefreshToken: void 0,
      IdToken: tokens.IdToken,
      NewDeviceMetadata: void 0,
      TokenType: void 0,
      ExpiresIn: void 0
    }
  };
};
const InitiateAuth = (services) => async (ctx, req) => {
  const userPool = await services.cognito.getUserPoolForClientId(ctx, req.ClientId);
  const userPoolClient = await services.cognito.getAppClient(ctx, req.ClientId);
  if (!userPoolClient) {
    throw new import_errors.NotAuthorizedError();
  }
  if (req.AuthFlow === "USER_PASSWORD_AUTH") {
    return userPasswordAuthFlow(ctx, req, userPool, userPoolClient, services);
  } else if (req.AuthFlow === "REFRESH_TOKEN" || req.AuthFlow === "REFRESH_TOKEN_AUTH") {
    return refreshTokenAuthFlow(ctx, req, userPool, userPoolClient, services);
  } else {
    throw new import_errors.UnsupportedError(`InitAuth with AuthFlow=${req.AuthFlow}`);
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  InitiateAuth
});
